from metrics_tools.interframe_psnr_metric import compute_interframe_psnr
from metrics_tools.global_psnr_metric import compute_global_psnr
from metrics_tools.crop_ratio_metric import calculate_cropped_pixel_ratio, get_crop_ratio
from metrics_tools.stability_metric import calculate_stability
from metrics_tools.distortion_metric import calculate_distortion
import cv2
import numpy as np
from skimage.metrics import peak_signal_noise_ratio as compute_psnr
from pipeline_tools.feature_extraction import extract_feature_keypoints_between_frames


def evaluate_metrics(original_frames, stabilized_frames, N):

    ITF = compute_interframe_psnr(stabilized_frames, N)

    GTF = compute_global_psnr(original_frames, stabilized_frames, N)

    CR = calculate_cropped_pixel_ratio(stabilized_frames, N)

    return ITF, GTF, CR 

def new_metrics(original_frames, stabilized_frames, N): 

    avg_ITF = [] 
    avg_GTF = []
    avg_CR = []

    total_pixels = original_frames[0].shape[0] * original_frames[0].shape[1]
    for i in range(N-1): 

        original_frame = original_frames[i]
        frame1 = stabilized_frames[i]
        frame2 = stabilized_frames[i+1]  

        #new_transform = new_transforms[i]
    
        ITF = compute_psnr(frame1, frame2 + 0.001) # add a little to prevent divide by zero if similar frames

        GTF = compute_psnr(original_frame, frame1 + 0.001) # add a little to prevent divide by zero if similar frames

        CR = get_crop_ratio(original_frame, frame1, total_pixels)

        avg_ITF.append(ITF)
        avg_GTF.append(GTF)
        avg_CR.append(CR)

    avg_ITF = np.mean(avg_ITF)
    avg_GTF = np.mean(avg_GTF)
    avg_CR = np.mean(avg_CR)

    return avg_ITF, avg_GTF, avg_CR