import numpy as np

from optimization_tools.l1_optimization import stabilize

from pipeline_tools.frame_generation import generate_frames
from pipeline_tools.feature_extraction import get_feature_transform_list_opt
from pipeline_tools.visualization import graph_optimized_paths
from pipeline_tools.video_reconstruction import frames_to_video


def optimization_script_weight_compare(video_name):

    w1s = [1,0,0]
    w2s = [0,1,0]
    w3s = [0,0,1]

    frames = generate_frames(video_name)

    N = len(frames)

    F_transforms, frame_shape = get_feature_transform_list_opt(frames, N)

    paths = [] 

    for i in range(len(w1s)):
        # Get stabilization transforms B_t by processing motion transition transforms F_t
        B_transforms = stabilize(F_transforms, frame_shape, True, None, crop_ratio=0.8, w1=w1s[i], w2=w2s[i], w3=w3s[i])
        # Accumulate by right multiplication into C_trajectory
        # $C_{t + 1} = C_t x F_t$
        # Initialise camera trajectory as a copy since it would help with stealing index 0 and the shape of F itself
        C_trajectory = F_transforms.copy()

        for i in range(1,N):
            # Right multiply transformations to accumulate all the changes into camera trajectory
            C_trajectory[i, :, :] = C_trajectory[i - 1, :, :] @ F_transforms[i, :, :]
        # Repeat right multiplication procedure to obtain stabilized camera trajectory P
        P_trajectory = C_trajectory.copy()
        # Apply transform to C_trajectory to get P_trajectory
        for i in range(N):
            P_trajectory[i, :, :] = C_trajectory[i, :, :] @ B_transforms[i, :, :]
        
        origin = np.array([0, 0, 1])

        evolution_og = origin @ C_trajectory

        evolution_stab = origin @ P_trajectory

        paths.append(evolution_stab)
    
    graph_optimized_paths(evolution_og, paths, N, 'l1_opt_comparison.png', w1s,w2s,w3s)

optimization_script_weight_compare('./videos/youtube_vid_1.mp4')