import numpy as np 


def get_stabilized_transforms_and_trajectory(original_transforms, trajectory, N, filtering_func, *args):

    dims = original_transforms.shape[1]

    smoothed_trajectory = filtering_func(trajectory, *args)

    difference = smoothed_trajectory - trajectory

    #Calculate newer transformation array
    transforms_smooth = original_transforms + difference

    smooth_transforms = np.zeros((N,3,3), np.float32)
    smooth_transforms[:,:,:] = np.eye(3)

    for i in range(N):
    
        #Extract transformations from the new transformation array
        dx = transforms_smooth[i, 0]
        dy = transforms_smooth[i, 1]
        if dims == 2: 
            dt = 0
        else:
            dt = transforms_smooth[i, 2]
        #Reconstruct transformation matrix accordingly to new values
        m = np.zeros((2, 3), np.float32)
        m[0, 0] = np.cos(dt)
        m[0, 1] = -np.sin(dt)
        m[1, 0] = np.sin(dt)
        m[1, 1] = np.cos(dt)
        m[0, 2] = dx
        m[1, 2] = dy

        smooth_transforms[i,:2,:] = m 

    return smooth_transforms, smoothed_trajectory 
