import cv2
import os
import numpy as np

def frames_to_video(new_frames, folder_path, output_video_path, fps=30):
    if not os.path.exists(folder_path): 
            os.makedirs(folder_path) 

    full_path = folder_path + output_video_path

    height, width, layers = new_frames[0].shape
    video = cv2.VideoWriter(full_path, cv2.VideoWriter_fourcc(*'mp4v'), fps, (width, height))

    i = 0 
    for frame in new_frames:
        video.write(frame)

        frame_filename = folder_path + 'frame_' + str(i) + '.jpg'
        cv2.imwrite(frame_filename, frame)

        i+=1 

    cv2.destroyAllWindows()
    video.release()

def crop_frames(frames, N, shape, crop_ratio=0.8): 

    new_frames = []
    new_frames.append(frames[0])

    for i in range(1,N):

        scale_x = 1 / crop_ratio
        scale_y = 1 / crop_ratio

        scaling_matrix = np.eye(3, dtype=np.float32)
        scaling_matrix[0][0] = scale_x
        scaling_matrix[1][1] = scale_y

        shifting_to_center_matrix = np.eye(3, dtype=np.float32)
        shifting_to_center_matrix[0][2] = -shape[1] / 2.0
        shifting_to_center_matrix[1][2] = -shape[0] / 2.0

        shifting_back_matrix = np.eye(3, dtype=np.float32)
        shifting_back_matrix[0][2] = shape[1] / 2.0
        shifting_back_matrix[1][2] = shape[0] / 2.0

        crop_matrix = shifting_back_matrix @ scaling_matrix @ shifting_to_center_matrix
        frame_stabilized = cv2.warpAffine(frames[i], crop_matrix[:2,:], (shape[1], shape[0]))
        new_frames.append(frame_stabilized)

    return new_frames 

def generate_new_frames(original_frames, B_transforms, N, shape, folder_path, filtered=False, crop_ratio=0.8):

    new_frames = []
    new_frames.append(original_frames[0])
    #res_frames = []

    # Write n_frames transformed frames
    for i in range(N):
        # Read the first/next frame
        frame = original_frames[i]

        # Apply affine wrapping to the given frame
        # Also convert to sta
        scale_x = 1 / crop_ratio
        scale_y = 1 / crop_ratio

        scaling_matrix = np.eye(3, dtype=np.float32)
        scaling_matrix[0][0] = scale_x
        scaling_matrix[1][1] = scale_y

        shifting_to_center_matrix = np.eye(3, dtype=np.float32)
        shifting_to_center_matrix[0][2] = -shape[1] / 2.0
        shifting_to_center_matrix[1][2] = -shape[0] / 2.0

        shifting_back_matrix = np.eye(3, dtype=np.float32)
        shifting_back_matrix[0][2] = shape[1] / 2.0
        shifting_back_matrix[1][2] = shape[0] / 2.0

        if filtered == True: 
            final_matrix = B_transforms[i]
            #final_matrix = affine_trans_info[i,:2,:] + B_transforms[i,:2,:]
            crop_matrix = shifting_back_matrix @ scaling_matrix @ shifting_to_center_matrix
            res_matrix = shifting_back_matrix @ scaling_matrix @ shifting_to_center_matrix
 
            frame_stabilized = cv2.warpAffine(frame, final_matrix[:2,:], (shape[1], shape[0]))
            frame_stabilized = cv2.warpAffine(frame_stabilized, crop_matrix[:2,:], (shape[1], shape[0]))
            #res_frame = cv2.warpAffine(frame, res_matrix[:2,:], (shape[1], shape[0]))
        else: 
            B_matrix = np.eye(3, dtype=np.float32)
            B_matrix[:2][:] = B_transforms[i, :, :2]
            final_matrix = shifting_back_matrix @ scaling_matrix @ shifting_to_center_matrix @ np.linalg.inv(B_matrix)

            frame_stabilized = cv2.warpAffine(frame, final_matrix[:2,:], (shape[1], shape[0]))
            #res_frame = cv2.warpAffine(frame, res_matrix[:2,:], (shape[1], shape[0]))
        # Save frames to an output folder 
        # if not os.path.exists(folder_path): 
        #     os.makedirs(folder_path) 

        # frame_filename = folder_path + 'frame_' + str(i) + '.jpg'
        # cv2.imwrite(frame_filename, frame_stabilized)
        new_frames.append(frame_stabilized)
        #res_frames.append(res_frame)

    return new_frames
  
  
def reconstruct_new_video(B_transforms, N, frames, folder_path, output_video_path, filtered=False): 
    
    print(f"Reconstructing video at {folder_path}{output_video_path}. Saving frames at {folder_path}...")

    frame_gray = cv2.cvtColor(frames[0], cv2.COLOR_BGR2GRAY)
    frame_shape = frame_gray.shape

    if filtered == True: 
        new_frames = generate_new_frames(frames, B_transforms, N, frame_shape, folder_path, filtered=True, crop_ratio=0.6)
    else:
        new_frames = generate_new_frames(frames, B_transforms, frame_shape, folder_path)

    # frames_to_video(new_frames, folder_path, output_video_path, fps=30)
    #frames_to_video(res_frame, folder_path, "original_cropped.mp4", fps=30)

    return new_frames