import numpy as np 
import matplotlib.pyplot as plt 
import cv2 
import os 

def graph_paths(original_path, filtered_path, N, filename): 

    # place data on the subplots
    fig, axs = plt.subplots(1,2, figsize=(10, 5))
    axs[0].set_title('x path')
    axs[0].plot(np.arange(0, N), original_path[:,0], '-r')
    axs[0].plot(np.arange(0, N), filtered_path[:,0], '-g')
    axs[1].set_title('y path')
    axs[1].plot(np.arange(0, N), original_path[:,1], '-r')
    axs[1].plot(np.arange(0, N), filtered_path[:,1], '-g')
    plt.savefig(filename)

def graph_optimized_paths(original_path, filtered_paths, N, filename, w1s, w2s, w3s): 

    # place data on the subplots
    fig, axs = plt.subplots(1,2,figsize=(10, 5))
    axs[0].set_title('x path')
    axs[0].plot(np.arange(0, N), original_path[:,0], label = 'original')
    axs[1].set_title('y path')
    axs[1].plot(np.arange(0, N), original_path[:,1],label = 'original')

    for i in range(len(filtered_paths)):
        cur_path = filtered_paths[i]
        axs[0].plot(np.arange(0, N), cur_path[:,0], label = f'w1 = {w1s[i]}, w2 = {w2s[i]}, w3 = {w3s[i]}')
        axs[1].plot(np.arange(0, N), cur_path[:,1], label = f'w1 = {w1s[i]}, w2 = {w2s[i]}, w3 = {w3s[i]}')

    plt.legend()
    plt.savefig(filename)

def graph_filtered_paths(original_path, filtered_paths, N, hyperparameters, filename, parameter_name): 

    # place data on the subplots
    fig, axs = plt.subplots(1,2,figsize=(10, 5))
    axs[0].set_title('x path')
    axs[0].plot(np.arange(0, N), original_path[:,0], label = 'original')
    axs[1].set_title('y path')
    axs[1].plot(np.arange(0, N), original_path[:,1],label = 'original')

    for h in hyperparameters:
        cur_path = filtered_paths[f'{h:.2f}']
        axs[0].plot(np.arange(0, N), cur_path[:,0], label = f'{parameter_name} = {h:.2f}')
        axs[1].plot(np.arange(0, N), cur_path[:,1], label = f'{parameter_name} = {h:.2f}')

    plt.legend()
    plt.savefig(filename)

def graph_comparison_filtered_paths(original_path, filtered_paths, N, filename): 

    # place data on the subplots
    fig, axs = plt.subplots(1,2,figsize=(10, 5))
    axs[0].set_title('x path')
    axs[0].plot(np.arange(0, N), original_path[:,0], linewidth=2, label = 'Original')
    axs[1].set_title('y path')
    axs[1].plot(np.arange(0, N), original_path[:,1], linewidth=2, label = 'Original')

    for path_name in filtered_paths:
        path = filtered_paths[path_name]
        axs[0].plot(np.arange(0, N), path[:,0], label = path_name)
        axs[1].plot(np.arange(0, N), path[:,1], label = path_name)

    plt.legend(fontsize='small')
    plt.savefig(filename)

def visualize_metrics_fine_tuning():
    conv_times = [times[f'lp_conv_{sigma}'] for sigma in sigmas]
    fft_times = [times[f'lp_fft_{sigma}'] for sigma in sigmas]
    fig, ax = plt.subplots()
    x = np.arange(2)
    width = 1/(len(conv_times) + 1)
    conv_ticks = np.linspace(0, 1, len(conv_times))
    fft_ticks = np.linspace(2, 3, len(fft_times))
    ax.bar(conv_ticks, conv_times, width, label='spatial domain')
    ax.bar(fft_ticks, fft_times, width, label='Fourier domain')
    ax.set_yscale('log')
    ax.set_xticks(np.concatenate((conv_ticks, fft_ticks), axis=0))
    ax.set_xticklabels(sigmas+sigmas)
    ax.set_xlabel('sigma')
    ax.legend()
    ax.set_ylabel('Runtimes in seconds')
    ax.set_title('Lowpass filter runtime comparison')
    fig.savefig('task1a_lowpass_times.png')

'''
def visualize_metrics_filtering_methods(ITF, GTF, crop_ratio):
    lowpass_itfs = ["Lowpass"] for i in ITF
    kalman_itfs = ["Lowpass"] for i in ITF
    lowpass_itfs = ["Lowpass"] for i in ITF

    conv_times = [times[f'lp_conv_{sigma}'] for sigma in sigmas]
    fft_times = [times[f'lp_fft_{sigma}'] for sigma in sigmas]
    fig, ax = plt.subplots()
    x = np.arange(2)
    width = 1/(len(conv_times) + 1)
    conv_ticks = np.linspace(0, 1, len(conv_times))
    fft_ticks = np.linspace(2, 3, len(fft_times))
    ax.bar(conv_ticks, conv_times, width, label='spatial domain')
    ax.bar(fft_ticks, fft_times, width, label='Fourier domain')
    ax.set_yscale('log')
    ax.set_xticks(np.concatenate((conv_ticks, fft_ticks), axis=0))
    ax.set_xticklabels(sigmas+sigmas)
    ax.set_xlabel('sigma')
    ax.legend()
    ax.set_ylabel('Runtimes in seconds')
    ax.set_title('Lowpass filter runtime comparison')
    fig.savefig('task1a_lowpass_times.png')
'''
# Function to display a slice of each frame
def display_frame_slices(frames, N, n_frames, folder_path, slice_height=20, a=3):

    print(f"Displaying frame slices at {folder_path}...")

    if not os.path.exists(folder_path): 
            os.makedirs(folder_path) 

    h = frames[0].shape[0]
    w = frames[0].shape[1]

    slice_begin = int(h - h/a) # slice at halfway point 

    # Read and process each frame
    for i in n_frames:

        # Extract the slice from each frame
        cur_frame = frames[i]

        slice_frame = cur_frame[slice_begin:slice_begin+slice_height, :, :]
        resized_slice_frame= cv2.resize(slice_frame, (w,h))
        slice_frame_filename = folder_path + 'sliced_frame_' + str(i) + '.jpg'
        cv2.imwrite(slice_frame_filename, resized_slice_frame)

        rerendered_frame = cur_frame
        rerendered_frame[slice_begin:slice_begin+slice_height, :, :] = [0, 255, 0] # Turn sliced pixels green
        rerendered_frame_filename = folder_path + 'rerendered_frame_' + str(i) + '.jpg'
        cv2.imwrite(rerendered_frame_filename, rerendered_frame)

